#!/usr/local/bin/ruby

$LOAD_PATH.unshift 'ext'

require 'test/unit'
require 'test/unit/ui/console/testrunner'
require 'openbase'


class TestOpenBase < Test::Unit::TestCase

  DATABASE = 'OBMovies'
  HOST     = 'localhost'
  LOGIN    = 'admin'
  PASSWORD = ''
  ENCODING = 'NSISOLatin1StringEncoding'

  def setup
    @ob = OpenBase.new(DATABASE, HOST, LOGIN, PASSWORD)
    @ob2 = OpenBase.new('OBOrderDatabase', HOST, 'admin', '')
    @ob3 = OpenBase.new('WORealEstate', HOST, 'admin', '')
  end

  def teardown
    @ob.invalidate
    @ob2.invalidate
    @ob3.invalidate
  end

  def test_connect
    assert_nothing_raised do
      ob = OpenBase.new(DATABASE, HOST, LOGIN, PASSWORD)
      ob.invalidate
    end

    assert_raise(OpenBaseError) do
      OpenBase.new('NotFoundDatabaseName', HOST, LOGIN, PASSWORD)
    end
  end

  def test_invalidate
    assert @ob.connect?
    @ob.invalidate
    assert_equal(false, @ob.connect?)
    assert_raise(OpenBaseError) { @ob.database }

    @ob.connect(DATABASE, HOST, LOGIN, PASSWORD)
    assert_nothing_raised { @ob.database }
  end

  def test_accessors
    assert_equal(ENCODING, @ob.encoding)
    assert_equal(DATABASE, @ob.database)
    assert_equal(HOST, @ob.host)
    assert_equal(LOGIN, @ob.login)
    assert_equal(PASSWORD, @ob.password)
  end

  def test_unique_row_id
    assert_not_nil @ob.unique_row_id('MOVIE', 'TITLE')
    assert_not_nil @ob.unique_row_id('MOVIE')
  end

  def test_execute
    result = @ob.execute 'SELECT TITLE, CATEGORY FROM MOVIE'
    assert_kind_of(OpenBaseResult, result)
  end

  def test_result_with_column_info
    result = @ob.execute 'select t0.TITLE, t0.REVENUE, t1.NAME from MOVIE t0, STUDIO t1 where t0.STUDIO_ID = t1.STUDIO_ID order by t0.REVENUE DESC'

    assert_equal(3, result.column_count)
    assert(result.rows_affected > 0)

    # column info
    info = result.column_infos[0]
    assert_equal('TITLE', info.name)
    assert_equal('char', info.type)
    assert_equal('MOVIE', info.table)
  end


  #
  # testsing transaction
  #

  def test_transaction
    id = 1000
    commit = 'Commit it'
    roll = 'Rollback it'

    # in progress
    @ob.begin
    assert @ob.transaction?
    @ob.rollback
    assert_equal(false, @ob.transaction?)

    # commit
    @ob.transaction do |handle|
      handle.execute "insert into MOVIE (MOVIE_ID, TITLE) values (#{id}, '#{commit}')"
    end

    result = @ob.execute "select MOVIE_ID, TITLE from MOVIE where MOVIE_ID = #{id}"
    row = result.fetch
    assert_equal([id, commit], row)
    @ob.execute "delete from MOVIE where MOVIE_ID = #{id}"

    # rollback
    assert_raise(RuntimeError) do
      @ob.transaction do |handle|
        handle.execute "insert into MOVIE (MOVIE_ID, TITLE) values (#{id}, '#{roll}')"
        raise "rollback it!"
      end
    end

    result = @ob.execute "select MOVIE_ID, TITLE from MOVIE where MOVIE_ID = #{id}"
    assert_nil result.fetch
  end


  #
  # testing each data types (10 types)
  #

  def test_type1_char
    result = @ob.execute 'SELECT TITLE FROM MOVIE'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end

  def test_type2_int
    result = @ob2.execute 'SELECT productShipped FROM orders'
    result.each do |row|
      assert_kind_of(Fixnum, row[0])
    end
    @ob2.invalidate
  end

  def test_type3_float
    result = @ob.execute 'SELECT RUNNING_AVERAGE FROM VOTING'
    result.each do |row|
      assert_kind_of(Float, row[0])
    end
  end

  def test_type4_long
    result = @ob.execute 'SELECT MOVIE_ID FROM MOVIE'
    result.each do |row|
      assert_kind_of(Integer, row[0])
    end
  end

  def test_type5_money
    result = @ob2.execute 'SELECT orderShipping FROM orders'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end

  def test_type6_date
    result = @ob2.execute 'SELECT orderDate FROM orders'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end

=begin
  def test_type7_time
    result = @ob.execute 'SELECT callTime FROM callHistory'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end
=end

  def test_type8_object
    result = @ob.execute 'SELECT PHOTO FROM TALENT_PHOTO'
    photo = false
    result.each do |row|
      assert_kind_of(String, row[0])
      unless row[0].empty?
        photo = true
      end
    end
    assert photo
  end

  def test_type9_datetime
    result = @ob.execute 'SELECT DATE_RELEASED FROM MOVIE'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end

  def test_type10_longlong
    result = @ob.execute 'SELECT _rowid FROM MOVIE'
    result.each do |row|
      assert_kind_of(Integer, row[0])
    end
  end

  def test_type11_boolean
    result = @ob3.execute 'SELECT ISSOLD FROM LISTING'
    result.each do |row|
      assert(row[0] == true || row[0] == false || row[0].nil?)
    end
  end

=begin
  def test_type12_binary
  end

  def test_type13_text
  end
=end

  def test_type14_timestamp
    result = @ob.execute 'SELECT _timestamp FROM MOVIE'
    result.each do |row|
      assert_kind_of(String, row[0])
    end
  end

end


if __FILE__ == $0 then
  suite = TestOpenBase.suite
  runner = Test::Unit::UI::Console::TestRunner.new(suite, 2)
  runner.start
end
